@@ -3,6 +3,7 @@
 
   Copyright (C) 2007 Donald Ephraim Curtis
   Copyright (C) 2008 Marcus D. Hanwell
+  Copyright (c) 2010 Konstantin Tokarev
 
   This file is part of the Avogadro molecular editor project.
   For more information, see <http://avogadro.openmolecules.net/>
@@ -37,30 +38,40 @@
 
 using Eigen::Vector3d;
 
- namespace Avogadro {
+namespace Avogadro {
 
-   class AtomPrivate {
-   public:
-     AtomPrivate(): assignedFormalCharge(false) {}
-     bool assignedFormalCharge;
-   };
+  class AtomPrivate {
+  public:
+    AtomPrivate(): assignedFormalCharge(false), groupIndex(0), residue(FALSE_ID),
+      partialCharge(0.0), formalCharge(0), forceVector(0.0, 0.0, 0.0),
+      customRadius(0.0)
+    {}
 
-   Atom::Atom(QObject *parent) : Primitive(AtomType, parent),
-                                 d_ptr(new AtomPrivate),
-                                 m_atomicNumber(0),
-                                 m_residue(FALSE_ID), m_partialCharge(0.0),
-                                 m_formalCharge(0),
-                                 m_forceVector(0.0, 0.0, 0.0)
-   {
-     if (!parent) {
-       qDebug() << "I am an orphaned atom! I feel so invalid...";
-     }
-     m_molecule = static_cast<Molecule*>(parent);
-   }
+    bool assignedFormalCharge;
+    unsigned int groupIndex;
+    unsigned long residue;
+    mutable double partialCharge;
+    int formalCharge;
+    Eigen::Vector3d forceVector;
+    QString customLabel;
+    QString customColorName;
+    double customRadius;
+  };
 
-   Atom::~Atom()
-   {
-   }
+  Atom::Atom(QObject *parent) : Primitive(AtomType, parent),
+                                d_ptr(new AtomPrivate),
+                                m_atomicNumber(0)
+  {
+    if (!parent) {
+      qDebug() << "I am an orphaned atom! I feel so invalid...";
+    }
+    m_molecule = static_cast<Molecule*>(parent);
+  }
+
+  Atom::~Atom()
+  {
+    delete d_ptr;
+  }
 
    const Eigen::Vector3d * Atom::pos() const
    {
@@ -129,25 +140,32 @@ using Eigen::Vector3d;
    double Atom::partialCharge() const
    {
      if (m_molecule && m_atomicNumber) {
+       Q_D(const Atom);
        m_molecule->calculatePartialCharges();
-       return m_partialCharge;
+       return d->partialCharge;
      }
      else
        return 0.0;
    }
 
+   void Atom::setPartialCharge(double charge) const
+   {
+     Q_D(const Atom);
+     d->partialCharge = charge;
+   }
+
    void Atom::setFormalCharge(int charge)
    {
      Q_D(Atom);
      d->assignedFormalCharge = true;
-     m_formalCharge = charge;
+     d->formalCharge = charge;
    }
 
    int Atom::formalCharge() const
    {
      Q_D(const Atom);
      if (d->assignedFormalCharge)
-       return m_formalCharge;
+       return d->formalCharge;
 
      // gotta guess it from bonding
      int valenceE = 0;
@@ -174,7 +192,7 @@ using Eigen::Vector3d;
        valenceE = atomicNum - 86;
      else
        return 0; // I don't quite know what to do for TM or other elements
-	
+
      int formalcharge = 0;
      int totalBonds = 0;
      foreach(unsigned long id, m_bonds) {
@@ -210,43 +228,135 @@ using Eigen::Vector3d;
        if (totalBonds < valenceE)
          loneE = fullShell - (2*totalBonds);
      }
-		
+
      formalcharge = valenceE - (totalBonds + loneE);
-	
+
      return formalcharge;
    }
 
    void Atom::setResidue(unsigned long id)
    {
-     m_residue = id;
+     Q_D(Atom);
+     d->residue = id;
    }
 
    void Atom::setResidue(const Residue *residue)
    {
-     m_residue = residue->id();
+     Q_D(Atom);
+     d->residue = residue->id();
+   }
+
+   unsigned int Atom::groupIndex() const
+   {
+     Q_D(const Atom);
+     m_molecule->calculateGroupIndices();
+     return d->groupIndex;
+   }
+
+   void Atom::setGroupIndex(unsigned int index)
+   {
+     Q_D(Atom);
+     d->groupIndex = index;
+   }
+
+   void Atom::setForceVector(const Eigen::Vector3d &force)
+   {
+     Q_D(Atom);
+     d->forceVector = force;
+   }
+
+   void Atom::setCustomLabel(const QString &label)
+   {
+     Q_D(Atom);
+     d->customLabel = label;
+   }
+
+   void Atom::setCustomColorName(const QString &name)
+   {
+     Q_D(Atom);
+     d->customColorName = name;
+   }
+
+   void Atom::setCustomRadius(const double radius)
+   {
+     Q_D(Atom);
+     d->customRadius = radius;
+   }
+
+   QString Atom::customLabel() const
+   {
+     Q_D(const Atom);
+     return d->customLabel;
+   }
+
+   QString Atom::customColorName() const
+   {
+     Q_D(const Atom);
+     return d->customColorName;
+   }
+
+   double Atom::customRadius() const
+   {
+     Q_D(const Atom);
+     return d->customRadius;
+   }
+
+   const Eigen::Vector3d Atom::forceVector() const
+   {
+     Q_D(const Atom);
+     return d->forceVector;
    }
 
    unsigned long Atom::residueId() const
    {
-     return m_residue;
+     Q_D(const Atom);
+     return d->residue;
    }
 
    Residue * Atom::residue() const
    {
-     return m_molecule->residueById(m_residue);
+     Q_D(const Atom);
+     return m_molecule->residueById(d->residue);
    }
 
    OpenBabel::OBAtom Atom::OBAtom()
    {
+     Q_D(Atom);
      // Need to copy all relevant data over to the OBAtom
      OpenBabel::OBAtom obatom;
+     OpenBabel::OBPairData *obproperty;
      const Vector3d *v = m_molecule->atomPos(m_id);
      obatom.SetVector(v->x(), v->y(), v->z());
      obatom.SetAtomicNum(m_atomicNumber);
-     obatom.SetFormalCharge(m_formalCharge);
+     obatom.SetPartialCharge(d->partialCharge);
+     obatom.SetFormalCharge(d->formalCharge);
+     obatom.SetId(m_id);
+
+     // Save custom label
+     if (!d->customLabel.isEmpty()) {
+       obproperty = new OpenBabel::OBPairData;
+       obproperty->SetAttribute("label");
+       obproperty->SetValue(d->customLabel.toAscii().data());
+       obatom.SetData(obproperty);
+     }
+
+     // Save custom color
+     if(!d->customColorName.isEmpty()) {
+       obproperty = new OpenBabel::OBPairData;
+       obproperty->SetAttribute("color");
+       obproperty->SetValue(d->customColorName.toAscii().data());
+       obatom.SetData(obproperty);
+     }
+
+     // Save custom radius
+     if (d->customRadius) {
+       obproperty = new OpenBabel::OBPairData;
+       obproperty->SetAttribute("radius");
+       obproperty->SetValue(QString::number(d->customRadius).toAscii().data());
+       obatom.SetData(obproperty);
+     }
 
      // Add dynamic properties as OBPairData
-     OpenBabel::OBPairData *obproperty;
      foreach(const QByteArray &propertyName, dynamicPropertyNames()) {
        obproperty = new OpenBabel::OBPairData;
        obproperty->SetAttribute(propertyName.data());
@@ -257,13 +367,56 @@ using Eigen::Vector3d;
      return obatom;
    }
 
+/*   const OpenBabel::OBAtom Atom::OBAtom() const
+   {
+     // Need to copy all relevant data over to the OBAtom
+     OpenBabel::OBAtom obatom;
+     OpenBabel::OBPairData *obproperty;
+     const Vector3d *v = m_molecule->atomPos(m_id);
+     obatom.SetVector(v->x(), v->y(), v->z());
+     obatom.SetAtomicNum(m_atomicNumber);
+     obatom.SetFormalCharge(m_formalCharge);
+
+     // Save custom label
+     if (!m_customLabel.isEmpty()) {
+       obproperty = new OpenBabel::OBPairData;
+       obproperty->SetAttribute("label");
+       obproperty->SetValue(m_customLabel.toAscii().data());
+       obatom.SetData(obproperty);
+     }
+
+     // Save custom color
+     if(!m_customColorName.isEmpty()) {
+       obproperty = new OpenBabel::OBPairData;
+       obproperty->SetAttribute("color");
+       obproperty->SetValue(m_customColorName.toAscii().data());
+       obatom.SetData(obproperty);
+     }
+
+     // Add dynamic properties as OBPairData
+     foreach(const QByteArray &propertyName, dynamicPropertyNames()) {
+       obproperty = new OpenBabel::OBPairData;
+       obproperty->SetAttribute(propertyName.data());
+       obproperty->SetValue(property(propertyName).toByteArray().data());
+       obatom.SetData(obproperty);
+     }
+
+     return obatom;
+   }*/
+
    bool Atom::setOBAtom(OpenBabel::OBAtom *obatom)
    {
+     Q_D(Atom);
      // Copy all needed OBAtom data to our atom
      m_molecule->setAtomPos(m_id, Vector3d(obatom->x(), obatom->y(), obatom->z()));
      m_atomicNumber = obatom->GetAtomicNum();
+     d->partialCharge = obatom->GetPartialCharge();
+
+     // #ifdef OPENBABEL_IS_NEWER_THAN_2_2_99
+     // m_customLabel = obatom->GetCustomLabel();
+     // #endif
      if (obatom->GetFormalCharge() != 0)
-       m_formalCharge = obatom->GetFormalCharge();
+       d->formalCharge = obatom->GetFormalCharge();
 
      // And add any generic data as QObject properties
      std::vector<OpenBabel::OBGenericData*> data;
@@ -273,6 +426,18 @@ using Eigen::Vector3d;
      data = obatom->GetAllData(OpenBabel::OBGenericDataType::PairData);
      for (j = data.begin(); j != data.end(); ++j) {
        property = static_cast<OpenBabel::OBPairData *>(*j);
+       if (property->GetAttribute() == "label") {
+         d->customLabel = property->GetValue().c_str();
+         continue;
+       }
+       if (property->GetAttribute() == "color") {
+         d->customColorName = property->GetValue().c_str();
+         continue;
+       }
+       if (property->GetAttribute() == "radius") {
+         d->customRadius = QString(property->GetValue().c_str()).toDouble();
+         continue;
+       }
        setProperty(property->GetAttribute().c_str(), property->GetValue().c_str());
      }
 
@@ -281,16 +446,21 @@ using Eigen::Vector3d;
 
    Atom& Atom::operator=(const Atom& other)
    {
+     Q_D(Atom);
      // Virtually everything here is invariant apart from the index and possibly id
      if (other.pos())
        m_molecule->setAtomPos(m_id, *other.pos());
      else
        qDebug() << "Atom position returned null.";
-     m_atomicNumber = other.m_atomicNumber;
-     m_formalCharge = other.m_formalCharge;
+
+     // Check first, because this will invalidate residue info
+     if (m_atomicNumber != other.m_atomicNumber)
+       m_atomicNumber = other.m_atomicNumber;
+     d->formalCharge = other.formalCharge();
+     d->customLabel = other.customLabel();
+     d->customColorName = other.customColorName();
+     d->customRadius = other.customRadius();
      return *this;
    }
 
  } // End namespace Avogadro
-
-#include "atom.moc"
