@@ -31,6 +31,7 @@
 #include <avogadro/molecule.h>
 #include <avogadro/cube.h>
 #include <avogadro/mesh.h>
+#include <avogadro/color3f.h>
 #include <avogadro/painterdevice.h>
 
 #include <QReadWriteLock>
@@ -78,23 +79,25 @@ namespace Avogadro {
     // Render the opaque surface if m_alpha is 1
     if (m_alpha >= 0.999) {
       if (m_mesh1) {
-        if (m_mesh1->stable()) {
+        if (m_mesh1->lock()->tryLockForRead()) {
           if (m_colored)
             pd->painter()->drawColorMesh(*m_mesh1, m_renderMode);
           else {
             pd->painter()->setColor(&m_posColor);
             pd->painter()->drawMesh(*m_mesh1, m_renderMode);
           }
+          m_mesh1->lock()->unlock();
         }
       }
       if (m_mesh2) {
-        if (m_mesh2->stable()) {
+        if (m_mesh2->lock()->tryLockForRead()) {
           if (m_colored)
             pd->painter()->drawColorMesh(*m_mesh2, m_renderMode);
           else {
             pd->painter()->setColor(&m_negColor);
             pd->painter()->drawMesh(*m_mesh2, m_renderMode);
           }
+          m_mesh2->lock()->unlock();
         }
       }
     }
@@ -110,7 +113,7 @@ namespace Avogadro {
     // Render the transparent surface if m_alpha is between 0 and 1.
     if (m_alpha > 0.001 && m_alpha < 0.999) {
       if (m_mesh1) {
-        if (m_mesh1->stable()) {
+        if (m_mesh1->lock()->tryLockForRead()) {
           if (m_colored) {
             pd->painter()->setColor(&m_posColor); // For transparency
             pd->painter()->drawColorMesh(*m_mesh1, m_renderMode);
@@ -119,10 +122,11 @@ namespace Avogadro {
             pd->painter()->setColor(&m_posColor);
             pd->painter()->drawMesh(*m_mesh1, m_renderMode);
           }
+          m_mesh1->lock()->unlock();
         }
       }
       if (m_mesh2) {
-        if (m_mesh2->stable()) {
+        if (m_mesh2->lock()->tryLockForRead()) {
           if (m_colored) {
             pd->painter()->setColor(&m_negColor); // For transparency
             pd->painter()->drawColorMesh(*m_mesh2, m_renderMode);
@@ -131,6 +135,7 @@ namespace Avogadro {
             pd->painter()->setColor(&m_negColor);
             pd->painter()->drawMesh(*m_mesh2, m_renderMode);
           }
+          m_mesh2->lock()->unlock();
         }
       }
     }
@@ -144,15 +149,17 @@ namespace Avogadro {
       renderMode = 2;
 
     if (m_mesh1) {
-      if (m_mesh1->stable()) {
+      if (m_mesh1->lock()->tryLockForRead()) {
         pd->painter()->setColor(&m_posColor);
         pd->painter()->drawMesh(*m_mesh1, renderMode);
+        m_mesh1->lock()->unlock();
       }
     }
     if (m_mesh2) {
-      if (m_mesh2->stable()) {
+      if (m_mesh2->lock()->tryLockForRead()) {
         pd->painter()->setColor(&m_negColor);
         pd->painter()->drawMesh(*m_mesh2, renderMode);
+        m_mesh2->lock()->unlock();
       }
     }
     if (m_drawBox)
@@ -283,6 +290,8 @@ namespace Avogadro {
       Cube *cube = m_molecule->cubeById(m_mesh1->cube());
       m_min = cube->min();
       m_max = cube->max();
+      if (m_mesh1->colors().size() == 0)
+          m_colored = false;
 
       // Enable the combo if appropriate for mapped color
       if (m_settingsWidget) {
@@ -413,13 +422,14 @@ namespace Avogadro {
   void SurfaceEngine::setMolecule(const Molecule *molecule)
   {
     Engine::setMolecule(molecule);
-
-    connect(m_molecule, SIGNAL(primitiveAdded(Primitive*)),
+    if (m_molecule) {
+      connect(m_molecule, SIGNAL(primitiveAdded(Primitive*)),
             this, SLOT(addPrimitive(Primitive*)));
-    connect(m_molecule, SIGNAL(primitiveUpdated(Primitive*)),
+      connect(m_molecule, SIGNAL(primitiveUpdated(Primitive*)),
             this, SLOT(updatePrimitive(Primitive*)));
-    connect(m_molecule, SIGNAL(primitiveRemoved(Primitive*)),
+      connect(m_molecule, SIGNAL(primitiveRemoved(Primitive*)),
             this, SLOT(removePrimitive(Primitive*)));
+    }
 
     updateOrbitalCombo();
   }
@@ -427,13 +437,14 @@ namespace Avogadro {
   void SurfaceEngine::setMolecule(Molecule *molecule)
   {
     Engine::setMolecule(molecule);
-
-    connect(m_molecule, SIGNAL(primitiveAdded(Primitive*)),
+    if (m_molecule) {
+      connect(m_molecule, SIGNAL(primitiveAdded(Primitive*)),
             this, SLOT(addPrimitive(Primitive*)));
-    connect(m_molecule, SIGNAL(primitiveUpdated(Primitive*)),
+      connect(m_molecule, SIGNAL(primitiveUpdated(Primitive*)),
             this, SLOT(updatePrimitive(Primitive*)));
-    connect(m_molecule, SIGNAL(primitiveRemoved(Primitive*)),
+      connect(m_molecule, SIGNAL(primitiveRemoved(Primitive*)),
             this, SLOT(removePrimitive(Primitive*)));
+    }
 
     updateOrbitalCombo();
   }
@@ -449,16 +460,28 @@ namespace Avogadro {
       settings.setValue("mesh1Id", static_cast<int>(m_mesh1->id()));
     if (m_mesh2)
       settings.setValue("mesh2Id", static_cast<int>(m_mesh2->id()));
-//    settings.setValue("posColor", m_posColor);
-//    settings.setValue("posColor", m_negColor);
+    settings.setValue("posColor/r", m_posColor.red());
+    settings.setValue("posColor/g", m_posColor.green());
+    settings.setValue("posColor/b", m_posColor.blue());
+    settings.setValue("negColor/r", m_negColor.red());
+    settings.setValue("negColor/g", m_negColor.green());
+    settings.setValue("negColor/b", m_negColor.blue());
   }
 
   void SurfaceEngine::readSettings(QSettings &settings)
   {
     Engine::readSettings(settings);
     m_alpha = settings.value("alpha", 0.5).toDouble();
-    m_posColor.setAlpha(m_alpha);
-    m_negColor.setAlpha(m_alpha);
+    // Default: Positive = blue
+    m_posColor.setFromRgba(settings.value("posColor/r", 0.0).toDouble(),
+                           settings.value("posColor/g", 0.0).toDouble(),
+                           settings.value("posColor/b", 1.0).toDouble(),
+                           m_alpha);
+    // Default: Negative = red
+    m_negColor.setFromRgba(settings.value("negColor/r", 1.0).toDouble(),
+                           settings.value("negColor/g", 0.0).toDouble(),
+                           settings.value("negColor/b", 0.0).toDouble(),
+                           m_alpha);
     m_renderMode = settings.value("renderMode", 0).toInt();
     m_colored = settings.value("colorMode", false).toBool();
     m_drawBox = settings.value("drawBox", false).toBool();
@@ -467,9 +490,13 @@ namespace Avogadro {
                                                     qulonglong(FALSE_ID)).toInt());
       m_mesh2 = m_molecule->meshById(settings.value("mesh2Id",
                                                     qulonglong(FALSE_ID)).toInt());
-      Cube *cube = m_molecule->cubeById(m_mesh1->cube());
-      m_min = cube->min();
-      m_max = cube->max();
+      if (m_mesh1) {
+        Cube *cube = m_molecule->cubeById(m_mesh1->cube());
+        m_min = cube->min();
+        m_max = cube->max();
+        if (m_mesh1->colors().size() == 0)
+          m_colored = false;
+      }
     }
   }
 
