@@ -26,15 +26,26 @@
 #include <avogadro/molecule.h>
 
 #include <openbabel/mol.h>
+#include <openbabel/obconversion.h>
 
-#include <QAction>
-#include <QString>
+#include <QtGui/QAction>
+#include <QtGui/QMessageBox>
+#include <QtCore/QString>
+#include <QtCore/QDebug>
+#include <QtCore/QTimer>
+
+#include <QtNetwork/QNetworkAccessManager>
+#include <QtNetwork/QNetworkReply>
 
 using namespace OpenBabel;
 
 namespace Avogadro {
 
-  MolecularPropertiesExtension::MolecularPropertiesExtension(QObject *parent) : Extension(parent), m_molecule(0), m_dialog(0)
+  MolecularPropertiesExtension::MolecularPropertiesExtension(QObject *parent) : Extension(parent),
+                                                                                m_molecule(0), m_dialog(0),
+                                                                                m_inchi(),
+                                                                                m_network(0),
+                                                                                m_nameRequestPending(false)
   {
     QAction *action = new QAction(this);
     action->setText(tr("Molecule Properties..."));
@@ -42,8 +53,7 @@ namespace Avogadro {
   }
 
   MolecularPropertiesExtension::~MolecularPropertiesExtension()
-  {
-  }
+  {  }
 
   QList<QAction *> MolecularPropertiesExtension::actions() const
   {
@@ -84,6 +94,11 @@ namespace Avogadro {
       connect(m_dialog, SIGNAL(accepted()), this, SLOT(disableUpdating()));
       connect(m_dialog, SIGNAL(rejected()), this, SLOT(disableUpdating()));
     }
+    if (!m_network) {
+      m_network = new QNetworkAccessManager(this);
+      connect(m_network, SIGNAL(finished(QNetworkReply*)),
+              this, SLOT(replyFinished(QNetworkReply*)));
+    }
 
     connect(m_molecule, SIGNAL(moleculeChanged()), this, SLOT(update()));
     connect(m_molecule, SIGNAL(primitiveAdded(Primitive *)),
@@ -107,6 +122,8 @@ namespace Avogadro {
     connect(m_molecule, SIGNAL(bondUpdated(Bond *)),
             this, SLOT(updateBonds(Bond*)));
 
+    m_dialog->nameLine->setText(tr("unknown", "Unknown molecule name"));
+
     update();
     m_dialog->show();
 
@@ -118,14 +135,25 @@ namespace Avogadro {
     if (m_dialog == NULL || m_molecule == NULL)
       return;
 
-    QString format("%L1"); // localized numbers
+    // used multiple times below
     OpenBabel::OBMol obmol = m_molecule->OBMol();
+
+    if (!m_nameRequestPending) {
+      m_nameRequestPending = true;
+      // Wait 250 msec before requesting to limit number of requests
+      // (Additional throttling is done by checking the InChI for each request)
+      qDebug() << "Requesting IUPAC name...";
+      QTimer::singleShot(250, this, SLOT(requestIUPACName()));
+    }
+
+    QString format("%L1"); // localized numbers
     m_dialog->molecularWeightLine->setText(format.arg(obmol.GetMolWt(), 0, 'f', 3));
 
     // Copied from Kalzium
     QString formula(obmol.GetSpacedFormula(1,"").c_str());
     formula.replace( QRegExp( "(\\d+)" ), "<sub>\\1</sub>" );
     m_dialog->formulaLine->setText(formula);
+    // we should actually handle charges with superscripts too (e.g., [SO4]-2)
 
     m_dialog->energyLine->setText(format.arg(m_molecule->energy(), 0, 'f', 3));
     bool estimate = true; // estimated dipole
@@ -164,15 +192,82 @@ namespace Avogadro {
   {
     update();
   }
-  
+
   void MolecularPropertiesExtension::disableUpdating()
   {
     // don't ask for more updates
     disconnect( m_molecule, 0, this, 0 );
   }
 
+  void MolecularPropertiesExtension::clearName()
+  {
+    if (m_dialog)
+      m_dialog->nameLine->setText(tr("unknown", "Unknown molecule name"));
+    if (m_molecule)
+      m_molecule->setProperty("name", QVariant()); // set an invalid name, since we don't have one
+  }
+
+  void MolecularPropertiesExtension::replyFinished(QNetworkReply *reply)
+  {
+    // Read in all the data
+    if (!reply->isReadable()) {
+      QMessageBox::warning(qobject_cast<QWidget*>(parent()),
+                           tr("Network Download Failed"),
+                           tr("Network timeout or other error."));
+      reply->deleteLater();
+      clearName();
+      return;
+    }
+
+    // check if the data came through
+    QByteArray data = reply->readAll();
+    if (data.contains("Error report") || data.contains("<h1>")) {
+      reply->deleteLater();
+      clearName();
+      return;
+    }
+
+    QString name = QString(data).trimmed().toLower();
+    if (!name.isEmpty()) {
+      m_dialog->nameLine->setText(name);
+      if (m_molecule) {
+        m_molecule->setProperty("name", QVariant(name)); // set the name for future use by other code
+      }
+    } else {
+      clearName();
+    }
+
+    reply->deleteLater();
+  }
+
+  void MolecularPropertiesExtension::requestIUPACName()
+  {
+    if (m_dialog == NULL || m_molecule == NULL)
+      return;
+
+    m_nameRequestPending = false;
+    OpenBabel::OBMol obmol = m_molecule->OBMol();
+
+    // Check if the molecule has changed,
+    // so we need to ask for a new name from the resolver
+    OBConversion conv;
+    conv.SetOutFormat("inchi"); // use a standard InChI key (which avoids issues with URL escaping)
+    QString inchi = QString::fromStdString(conv.WriteString(&obmol, true)); // skip whitespace
+    if (m_inchi == inchi)
+      return; // no need to send a new query, since it's the same request
+
+    m_inchi = inchi; // cache for next use
+
+    QString requestURL = QLatin1String("http://cactus.nci.nih.gov/chemical/structure/") + m_inchi + QLatin1String("/iupac_name");
+    qDebug() << " requesting URL: " << requestURL;
+
+    m_network->get(QNetworkRequest(QUrl(requestURL)));
+
+    m_dialog->nameLine->setText(tr("(pending)", "asking server for molecule name"));
+  }
+
 } // end namespace Avogadro
 
-Q_EXPORT_PLUGIN2(molecularpropertiesextension,
+Q_EXPORT_PLUGIN2(molecularpropextension,
                  Avogadro::MolecularPropertiesExtensionFactory)
 
