@@ -2,6 +2,7 @@
   propextension.h - Properties Plugin for Avogadro
 
   Copyright (C) 2007-2008 by Tim Vandermeersch
+  Some portions Copyright (C) 2009 by Konstantin Tokarev
 
   This file is part of the Avogadro molecular editor project.
   For more information, see <http://avogadro.openmolecules.net/>
@@ -27,17 +28,21 @@
 #include <avogadro/bond.h>
 #include <avogadro/primitivelist.h>
 
-#include <QAbstractTableModel>
-#include <QSortFilterProxyModel>
-#include <QHeaderView>
-#include <QAction>
-#include <QDialog>
-#include <QVBoxLayout>
+#include <QtCore/QAbstractTableModel>
+#include <QtGui/QSortFilterProxyModel>
+#include <QtGui/QSizePolicy>
+#include <QtGui/QHeaderView>
+#include <QtGui/QAction>
+#include <QtGui/QDialog>
+#include <QtGui/QScrollBar>
+#include <QtGui/QVBoxLayout>
 
-#include <QDebug>
+#include <QtCore/QDebug>
 
 using namespace std;
 using namespace OpenBabel;
+using OpenBabel::OBGenericDataType::AngleData;
+using OpenBabel::OBGenericDataType::TorsionData;
 
 namespace Avogadro
 {
@@ -47,7 +52,7 @@ namespace Avogadro
     BondPropIndex,
     AnglePropIndex,
     TorsionPropIndex,
-    CartesianIndex,
+    //CartesianIndex,
     ConformerIndex
   };
 
@@ -85,10 +90,10 @@ namespace Avogadro
     action->setData(ConformerIndex);
     m_actions.append( action );
 
-    action = new QAction( this );
+    /*action = new QAction( this );
     action->setText( tr("Cartesian Editor..." ));
     action->setData(CartesianIndex);
-    m_actions.append( action );
+    m_actions.append( action );*/
   }
 
   PropertiesExtension::~PropertiesExtension()
@@ -111,10 +116,6 @@ namespace Avogadro
     case TorsionPropIndex:
     case ConformerIndex:
       return tr("&View") + '>' + tr("&Properties");
-    case CartesianIndex:
-    default:
-      return tr("&Build");
-      break;
     };
     return QString();
   }
@@ -132,6 +133,7 @@ namespace Avogadro
     PropertiesView  *view;
     QDialog *dialog = new QDialog(qobject_cast<QWidget *>(parent()));
     QVBoxLayout *layout = new QVBoxLayout(dialog);
+    dialog->setLayout(layout);
     // Don't show whitespace around the PropertiesView
     layout->setSpacing(0);
     layout->setContentsMargins(0,0,0,0);
@@ -174,7 +176,7 @@ namespace Avogadro
       // view will delete itself in PropertiesView::hideEvent using deleteLater().
       view = new PropertiesView(PropertiesView::TorsionType, widget);
       break;
-    case CartesianIndex: // cartesian editor
+    /*case CartesianIndex: // cartesian editor
       // m_angleModel will be deleted in PropertiesView::hideEvent using deleteLater().
       model = new PropertiesModel(PropertiesModel::CartesianType);
       model->setMolecule( m_molecule );
@@ -182,7 +184,7 @@ namespace Avogadro
       view = new PropertiesView(PropertiesView::CartesianType, widget);
       connect(m_molecule, SIGNAL(atomAdded(Atom*)), model, SLOT( atomAdded(Atom*)));
       connect(m_molecule, SIGNAL(atomRemoved(Atom*)), model, SLOT(atomRemoved(Atom*)));
-      break;
+      break;*/
     case ConformerIndex: // conformers
       // model will be deleted in PropertiesView::hideEvent using deleteLater().
       model = new PropertiesModel(PropertiesModel::ConformerType, dialog);
@@ -209,12 +211,21 @@ namespace Avogadro
     view->setMolecule( m_molecule );
     view->setWidget( widget );
     view->setModel( proxyModel );
-
+    view->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    view->resizeColumnsToContents();
     layout->addWidget(view);
     dialog->setWindowTitle(view->windowTitle());
     QSize dialogSize = dialog->size();
-    dialogSize.setWidth(550);
+    double width = view->horizontalHeader()->length()+view->verticalHeader()->width()+5;
+	if (model->rowCount() < 13) { // no scrollbar
+	  dialogSize.setHeight(view->horizontalHeader()->height()+model->rowCount()*30+5);
+      dialogSize.setWidth(width);
+    } else { // scrollbar is needed
+      dialogSize.setHeight(width/1.618);
+      dialogSize.setWidth(width+view->verticalScrollBar()->width());
+    }
     dialog->resize(dialogSize);
+	//dialog->setWindowFlags(Qt::Window);
     dialog->show();
 
     return undo;
@@ -239,9 +250,9 @@ namespace Avogadro
     case TorsionType:
       title = tr("Torsion Properties");
       break;
-    case CartesianType:
+    /*case CartesianType:
       title = tr("Cartesian Properties");
-      break;
+      break;*/
     case ConformerType:
       title = tr("Conformer Properties");
       break;
@@ -251,9 +262,12 @@ namespace Avogadro
     this->setWindowTitle(title);
 
     QHeaderView *horizontal = this->horizontalHeader();
-    horizontal->setResizeMode(QHeaderView::Stretch);
+    horizontal->setResizeMode(QHeaderView::Interactive);
+    horizontal->setMinimumSectionSize(75);
     QHeaderView *vertical = this->verticalHeader();
-    vertical->setResizeMode(QHeaderView::Stretch);
+    vertical->setResizeMode(QHeaderView::Interactive);
+    vertical->setMinimumSectionSize(30);
+    vertical->setDefaultAlignment(Qt::AlignCenter);
 
     // Don't allow selecting everything
     setCornerButtonEnabled(false);
@@ -267,16 +281,20 @@ namespace Avogadro
   void PropertiesView::selectionChanged(const QItemSelection &selected, const QItemSelection &)
   {
     QList<Primitive *> matchedPrimitives;
+    bool ok = false;
 
     foreach (const QModelIndex &index, selected.indexes()) {
       if (!index.isValid())
         return;
-
-      if (m_type == AtomType || m_type == CartesianType) {
+      int rowNum = model()->headerData(index.row(), Qt::Vertical).toString().split(" ").last().toLong(&ok) - 1;
+      if (!ok)
+        return;
+      
+      if (m_type == AtomType /*|| m_type == CartesianType*/) {
         if ((unsigned int) index.row() >= m_molecule->numAtoms())
           return;
 
-        matchedPrimitives.append( m_molecule->atom(index.row()) );
+        matchedPrimitives.append( m_molecule->atom(rowNum) );
         m_widget->clearSelected();
         m_widget->setSelected(matchedPrimitives, true);
         m_widget->update();
@@ -284,32 +302,60 @@ namespace Avogadro
         if((unsigned int) index.row() >= m_molecule->numBonds())
           return;
 
-        matchedPrimitives.append( m_molecule->bond(index.row()) );
+        matchedPrimitives.append( m_molecule->bond(rowNum) );
         m_widget->clearSelected();
         m_widget->setSelected(matchedPrimitives, true);
         m_widget->update();
       } else if (m_type == AngleType && model() != 0) {
-        int aIndex = model()->data(index.sibling(index.row(), 0)).toInt();
-        int bIndex = model()->data(index.sibling(index.row(), 1)).toInt();
-        int cIndex = model()->data(index.sibling(index.row(), 2)).toInt();
+        OBMol *mol = new OBMol(m_molecule->OBMol());
+        mol->FindAngles();
+        OBAngleData *ad = static_cast<OBAngleData *>(mol->GetData(AngleData));
+        if (!ad)
+          return;
+        vector<vector<unsigned int> > angles;
+        ad->FillAngleArray(angles);
+        delete mol;
+
+        Atom *startAtom = m_molecule->atom((angles[rowNum][1]));
+        Atom *vertex = m_molecule->atom((angles[rowNum][0]));
+        Atom *endAtom = m_molecule->atom((angles[rowNum][2]));
+        Bond *bond1 = startAtom->bond(vertex);
+        Bond *bond2 = vertex->bond(endAtom);
         
-        matchedPrimitives.append( m_molecule->atom( aIndex - 1) );
-        matchedPrimitives.append( m_molecule->atom( bIndex - 1) );
-        matchedPrimitives.append( m_molecule->atom( cIndex - 1) );
+        matchedPrimitives.append( startAtom );
+        matchedPrimitives.append( vertex );
+        matchedPrimitives.append( endAtom );
+        matchedPrimitives.append( bond1 );
+        matchedPrimitives.append( bond2 );
 
         m_widget->clearSelected();
         m_widget->setSelected(matchedPrimitives, true);
         m_widget->update();
       } else if (m_type == TorsionType && model() != 0) {
-        int aIndex = model()->data(index.sibling(index.row(), 0)).toInt();
-        int bIndex = model()->data(index.sibling(index.row(), 1)).toInt();
-        int cIndex = model()->data(index.sibling(index.row(), 2)).toInt();
-        int dIndex = model()->data(index.sibling(index.row(), 3)).toInt();
+        OBMol *mol = new OBMol(m_molecule->OBMol());
+        mol->FindTorsions();
+        OBTorsionData *td = static_cast<OBTorsionData *>(mol->GetData(TorsionData));
+        if (!td)
+          return;
+        vector<vector<unsigned int> > torsions;
+        td->FillTorsionArray(torsions);
+        delete mol;
+
+        Atom *a = m_molecule->atom( torsions[rowNum][0] );
+        Atom *b = m_molecule->atom( torsions[rowNum][1] );
+        Atom *c = m_molecule->atom( torsions[rowNum][2] );
+        Atom *d = m_molecule->atom( torsions[rowNum][3] );
+        Bond *bond1 = a->bond(b);
+        Bond *bond2 = b->bond(c);
+        Bond *bond3 = c->bond(d);
         
-        matchedPrimitives.append( m_molecule->atom( aIndex - 1) );
-        matchedPrimitives.append( m_molecule->atom( bIndex - 1) );
-        matchedPrimitives.append( m_molecule->atom( cIndex - 1) );
-        matchedPrimitives.append( m_molecule->atom( dIndex - 1) );
+        matchedPrimitives.append(a);
+        matchedPrimitives.append(b);
+        matchedPrimitives.append(c);
+        matchedPrimitives.append(d);
+        matchedPrimitives.append(bond1);
+        matchedPrimitives.append(bond2);
+        matchedPrimitives.append(bond3);
 
         m_widget->clearSelected();
         m_widget->setSelected(matchedPrimitives, true);
@@ -318,11 +364,10 @@ namespace Avogadro
         if (index.row() >= static_cast<int>(m_molecule->numConformers()))
           return;
 
-        m_molecule->setConformer(index.row());
+        m_molecule->setConformer(rowNum);
         m_molecule->update();
         return;
       }
-      // TODO: Highlight angles and torsions
     }
   }
 
@@ -338,12 +383,10 @@ namespace Avogadro
 
   void PropertiesView::hideEvent(QHideEvent *)
   {
-    if (m_widget)
+    if ((m_widget) && model()) {
       m_widget->clearSelected();
-
-    QAbstractItemModel *m_model = model();
-    if (m_model)
-      m_model->deleteLater();
+      model()->deleteLater();
+    }
 
     this->deleteLater();
   }
