@@ -40,7 +40,7 @@ using namespace boost::python;
 namespace Avogadro
 {
 
-  PythonTerminal::PythonTerminal( QObject *parent ) : Extension( parent ), m_molecule(0), m_terminalDock(0)
+  PythonTerminal::PythonTerminal( QObject *parent ) : DockExtension( parent ), m_molecule(0), m_terminalDock(0)
   {
   }
 
@@ -51,7 +51,7 @@ namespace Avogadro
       //m_terminalDock->deleteLater();
   }
 
-  QList<QAction *> PythonTerminal::actions() const
+  /*QList<QAction *> PythonTerminal::actions() const
   {
     return QList<QAction*>();
   }
@@ -61,7 +61,7 @@ namespace Avogadro
   {
     Q_UNUSED(action);
     return QString();
-  }
+  }*/
 
   QDockWidget * PythonTerminal::dockWidget()
   {
@@ -84,12 +84,12 @@ namespace Avogadro
     m_terminalEdit->setMolecule(molecule);
   }
 
-  QUndoCommand* PythonTerminal::performAction( QAction *action, GLWidget *widget )
+  /*QUndoCommand* PythonTerminal::performAction( QAction *action, GLWidget *widget )
   {
     Q_UNUSED(action);
     Q_UNUSED(widget);
     return 0;
-  }
+  }*/
 
   /*
   PythonTerminalWidget::PythonTerminalWidget( QWidget *parent ) : QWidget(parent)
@@ -135,7 +135,7 @@ namespace Avogadro
     QFont font;
     font.setFamily(QString::fromUtf8("Courier New"));
     setFont(font);
-         
+    m_interpreter.exec("import Avogadro");
     printPrompt();
 
 //    connect(this, SIGNAL(cursorPositionChanged()), this, SLOT(setTextCursorToEnd()));
@@ -173,11 +173,11 @@ namespace Avogadro
  
       if (line.endsWith(':')) {
         indent += 2;
-        m_lines.append(text);
+        m_lines.append(text + "\n");
         append(QLatin1String(""));
       } else {
         if (indent && !m_lines.isEmpty()) {
-          m_lines.append(text);
+          m_lines.append(text + "\n");
           append(QLatin1String(""));
         } else {
           QString result = m_interpreter.exec(text);
@@ -272,7 +272,9 @@ namespace Avogadro
       }
       event->accept();
       return;
-    } else if(event->key() == Qt::Key_Return) {
+    }
+    else if(event->key() == Qt::Key_Return)
+    {
       QString text = toPlainText();
       QString t = text.right(text.size() - m_cursorPos);
 
