@@ -27,7 +27,6 @@
  **********************************************************************/
 
 #include "plotwidget.h"
-#include "plotwidget.moc"
 
 #include <math.h>
 #include <QDebug>
@@ -61,7 +60,8 @@ namespace Avogadro {
       : q( qq ),
         cBackground( Qt::black ), cForeground( Qt::white ), cGrid( Qt::gray ),
         showGrid( false ), showObjectToolTip( true ), useAntialias( false ),
-        font( QFont() ), followingMouse(false)
+        font( QFont() ), followingMouse(false), jailedInDefaults(false),
+        labelShiftDirection(None)
     {
       // create the axes and setting their default properties
       PlotAxis *leftAxis = new PlotAxis();
@@ -93,6 +93,7 @@ namespace Avogadro {
     PlotWidget *q;
 
     void calcDataRectLimits( double x1, double x2, double y1, double y2 );
+    void calcDataRectLimits( QRectF & rect);
     /**
      * @return a value indicating how well the given rectangle is
      * avoiding masked regions in the plot.  A higher returned value
@@ -121,12 +122,15 @@ namespace Avogadro {
     QRectF dataRect, secondDataRect, defaultDataRect;
     // Limits of the plot area in pixel units
     QRect pixRect;
-    //Array holding the mask of "used" regions of the plot
+    // Array holding the mask of "used" regions of the plot
     QImage plotMask;
-    //Font properties
+    // Font properties
     QFont font;
-    //Whether to highlight the point nearest the mouse
+    // Whether to highlight the point nearest the mouse
     bool followingMouse;
+    // Wether can move away default limits rectangle
+    bool jailedInDefaults;
+    Direction labelShiftDirection;
   };
 
   PlotWidget::PlotWidget( QWidget * parent )
@@ -168,6 +172,12 @@ namespace Avogadro {
     update();
   }
 
+  void PlotWidget::setLimits( QRectF & rect)
+  {
+    d->calcDataRectLimits(rect);
+    update();
+  }
+
   void PlotWidget::scaleLimits(PlotObject * po) {
     double xmin=0, xmax=0, ymin=0, ymax=0;
     if (po) {
@@ -225,6 +235,12 @@ namespace Avogadro {
     setLimits( x1, x2, y1, y2 );
   }
 
+  void PlotWidget::setDefaultLimits( QRectF & rect)
+  {
+    d->defaultDataRect = rect;
+    setLimits(rect);
+  }
+
   void PlotWidget::unsetDefaultLimits()
   {
     if (!defaultDataRect().isNull()) {
@@ -263,6 +279,19 @@ namespace Avogadro {
     }
   }
 
+  void PlotWidget::Private::calcDataRectLimits( QRectF & rect)
+  {
+    dataRect = rect;
+    q->axis( LeftAxis )->setTickMarks( dataRect.y(), dataRect.height() );
+    q->axis( BottomAxis )->setTickMarks( dataRect.x(), dataRect.width() );
+
+    if ( secondDataRect.isNull() )
+    {
+      q->axis( RightAxis )->setTickMarks( dataRect.y(), dataRect.height() );
+      q->axis( TopAxis )->setTickMarks( dataRect.x(), dataRect.width() );
+    }
+  }
+
   void PlotWidget::setSecondaryLimits( double x1, double x2, double y1, double y2 ) {
     // Again, removed limit checking. Just be careful with it ;)
     double XA1, XA2, YA1, YA2;
@@ -356,6 +385,22 @@ namespace Avogadro {
     update();
   }
 
+  void PlotWidget::setJailedInDefaults(bool b)
+  {
+    d->jailedInDefaults = b;
+    // TODO: refactor shared code with mouseDoubleClick
+    double x1 = defaultDataRect().x();
+    double x2 = x1 + defaultDataRect().width();
+    double y1 = defaultDataRect().y();
+    double y2 = y1 + defaultDataRect().height();
+    setLimits(x1, x2, y1, y2);
+  }
+
+  void PlotWidget::setLabelShiftDirection(Direction dir, float /*priority*/)
+  {
+    d->labelShiftDirection = dir;
+  }
+
   void PlotWidget::addPlotObject( PlotObject *object )
   {
     // skip null pointers
@@ -585,8 +630,31 @@ namespace Avogadro {
       float newY1 = dataRect().y() + unitDelta.y();
       float newY2 = dataRect().y() + unitDelta.y() + dataRect().height();
 
-      setLimits(newX1, newX2, newY1, newY2);// Update axis
-
+      if (d->jailedInDefaults) {
+        if (defaultDataRect().width() > 0) {
+          if (newX1 < defaultDataRect().left() || (newX2 > defaultDataRect().right())) {
+            newX1 = dataRect().x();
+            newX2 = dataRect().x() + dataRect().width();
+          }
+        } else {
+          if (newX1 > defaultDataRect().left() || (newX2 < defaultDataRect().right())) {
+            newX1 = dataRect().x();
+            newX2 = dataRect().x() + dataRect().width();
+          }            
+        }
+        if (defaultDataRect().height() > 0) {
+          if (newY1 < defaultDataRect().top() || (newY2 > defaultDataRect().bottom())) {
+            newY1 = dataRect().y();
+            newY2 = dataRect().y() + dataRect().height();
+          }
+        } else {
+          if (newY1 < defaultDataRect().bottom() || (newY2 > defaultDataRect().top())){
+            newY1 = dataRect().y();
+            newY2 = dataRect().y() + dataRect().height();
+          }
+        }
+      }      
+      setLimits(newX1, newX2, newY1, newY2);
       mouseClickOrigin = event->posF();
     }
 
@@ -606,11 +674,9 @@ namespace Avogadro {
       QPointF pF 	= mapToWidget(mapFrameToData(event->pos()));
       QPoint p_widget 	( static_cast<int>(pF.x()), static_cast<int>(pF.y()));
       PlotPoint *p 	= pointNearestPoint(p_widget);
-      PlotPoint *old;
 
       if (p) { // Make sure p isn't null...
         if (d->mousefollow->points().size() != 0) {
-          old 	= d->mousefollow->at(0);
           d->mousefollow->clearPoints();
         }
         d->mousefollow->addPoint(p->x(), p->y());
@@ -641,6 +707,7 @@ namespace Avogadro {
   void PlotWidget::mouseDoubleClickEvent(QMouseEvent *event)
   {
     if ((event->buttons() & Qt::LeftButton) && !defaultDataRect().isNull()) {
+      // TODO: refactor shared code with setJailedInDefaults
       double x1 = defaultDataRect().x();
       double x2 = x1 + defaultDataRect().width();
       double y1 = defaultDataRect().y();
@@ -847,7 +914,23 @@ namespace Avogadro {
     QRectF bestRect = fm.boundingRect( QRectF( pos.x(), pos.y(), 1, 1 ), textFlags, pp->label() );
     float xStep = 0.5*bestRect.width();
     float yStep = 0.5*bestRect.height();
-    float maxCost = 0.05 * bestRect.width() * bestRect.height();
+     switch(d->labelShiftDirection) {
+      case Up:        
+        bestRect = fm.boundingRect( QRectF( pos.x(), pos.y()-3*yStep, 1, 1 ), textFlags, pp->label() );
+        break;
+      case Down:
+        bestRect = fm.boundingRect( QRectF( pos.x(), pos.y()+3*yStep, 1, 1 ), textFlags, pp->label() );
+        break;
+      case Left:
+        bestRect = fm.boundingRect( QRectF( pos.x()-3*xStep, pos.y(), 1, 1 ), textFlags, pp->label() );
+        break;
+      case Right:
+        bestRect = fm.boundingRect( QRectF( pos.x()+3*xStep, pos.y(), 1, 1 ), textFlags, pp->label() );
+        break;
+      default:
+        bestRect = fm.boundingRect( QRectF( pos.x(), pos.y(), 1, 1 ), textFlags, pp->label() );
+    }
+    float maxCost = 0.01 * bestRect.width() * bestRect.height();
     float bestCost = d->rectCost( bestRect );
 
     //We will travel along a path defined by the maximum decrease in
@@ -862,6 +945,7 @@ namespace Avogadro {
     int iter = 0;
     QList<int> TriedPathIndex;
     float bestBadCost = 10000;
+    float upCost=100000, downCost=100000, leftCost=100000, rightCost=100000;
     QRectF bestBadRect;
 
     //needed to halt iteration from inside the switch
@@ -872,16 +956,22 @@ namespace Avogadro {
       //step provides the lowest cost
       QRectF upRect = bestRect;
       upRect.moveTop( upRect.top() + yStep );
-      float upCost = d->rectCost( upRect );
+      if (d->labelShiftDirection != Down)
+        upCost = d->rectCost( upRect );        
       QRectF downRect = bestRect;
       downRect.moveTop( downRect.top() - yStep );
-      float downCost = d->rectCost( downRect );
+      if (d->labelShiftDirection != Up)
+        downCost = d->rectCost( downRect );
+      //else
+      //  qDebug() << "no down!" << upCost << downCost << leftCost << rightCost;
       QRectF leftRect = bestRect;
       leftRect.moveLeft( leftRect.left() - xStep );
-      float leftCost = d->rectCost( leftRect );
+      if (d->labelShiftDirection != Right)
+        leftCost = d->rectCost( leftRect );
       QRectF rightRect = bestRect;
       rightRect.moveLeft( rightRect.left() + xStep );
-      float rightCost = d->rectCost( rightRect );
+      if (d->labelShiftDirection != Left)
+        rightCost = d->rectCost( rightRect );
 
       //which direction leads to the lowest cost?
       QList<float> costList;
@@ -919,6 +1009,7 @@ namespace Avogadro {
       case 1: //down
         bestRect.moveTop( downRect.top() );
         bestCost = downCost;
+        //qDebug() << "down" << downCost;
         break;
       case 2: //left
         bestRect.moveLeft( leftRect.left() );
@@ -934,6 +1025,7 @@ namespace Avogadro {
           bestBadCost = bestCost;
           bestBadRect = bestRect;
         }
+        //qDebug() << "min" << pp->label() << bestCost;
 
         //If all of the first-step paths have now been searched, we'll
         //have to adopt the bestBadRect
@@ -946,7 +1038,23 @@ namespace Avogadro {
         //If we haven't yet tried all of the first-step paths, start over
         if ( TriedPathIndex.size() < 4 ) {
           iter = -1; //anticipating the ++iter below
-          bestRect = fm.boundingRect( QRectF( pos.x(), pos.y(), 1, 1 ), textFlags, pp->label() );
+    // TODO: remove code duplication
+     switch(d->labelShiftDirection) {
+      case Up:        
+        bestRect = fm.boundingRect( QRectF( pos.x(), pos.y()-3*yStep, 1, 1 ), textFlags, pp->label() );
+        break;
+      case Down:
+        bestRect = fm.boundingRect( QRectF( pos.x(), pos.y()+3*yStep, 1, 1 ), textFlags, pp->label() );
+        break;
+      case Left:
+        bestRect = fm.boundingRect( QRectF( pos.x()-3*xStep, pos.y(), 1, 1 ), textFlags, pp->label() );
+        break;
+      case Right:
+        bestRect = fm.boundingRect( QRectF( pos.x()+3*xStep, pos.y(), 1, 1 ), textFlags, pp->label() );
+        break;
+      default:
+        bestRect = fm.boundingRect( QRectF( pos.x(), pos.y(), 1, 1 ), textFlags, pp->label() );
+    }
           bestCost = d->rectCost( bestRect );
         }
         break;
@@ -962,19 +1070,24 @@ namespace Avogadro {
       ++iter;
     }
 
+    QPen oldpen = painter->pen();      
+    QPen pen(oldpen);
+    pen.setColor(d->cForeground);
+    painter->setPen( pen );
+      
     //Place label
     painter->drawText( bestRect, textFlags, pp->label() );
 
     //Is a line needed to connect the label to the point?
-    float deltax = pos.x() - bestRect.center().x();
-    float deltay = pos.y() - bestRect.center().y();
-    float rbest = sqrt( deltax*deltax + deltay*deltay );
-    if ( rbest > 20.0 ) {
+    //float deltax = pos.x() - bestRect.center().x();
+    //float deltay = pos.y() - bestRect.center().y();
+    //float rbest = sqrt( deltax*deltax + deltay*deltay );
+    //if ( rbest > 20.0 ) {
+    //if (fabs(deltax) <= 4*xStep && fabs(deltay) <= 4*yStep) {
       //Draw a rectangle around the label
       painter->setBrush( QBrush() );
-      //QPen pen = painter->pen();
+
       //pen.setStyle( Qt::DotLine );
-      //painter->setPen( pen );
       painter->drawRoundRect( bestRect );
 
       //Now connect the label to the point with a line.
@@ -992,7 +1105,8 @@ namespace Avogadro {
         yline = bestRect.bottom();
 
       painter->drawLine( QPointF( xline, yline ), pos );
-    }
+    //}
+    painter->setPen( oldpen );
 
     //Mask the label's rectangle so other labels won't overlap it.
     maskRect( bestRect );
