@@ -38,10 +38,6 @@
 #include <avogadro/extension.h>
 #include <avogadro/color.h>
 
-// Include static headers
-#include "engines/bsdyengine.h"
-#include "colors/elementcolor.h"
-
 #include <QSettings>
 #include <QDir>
 #include <QList>
@@ -50,6 +46,9 @@
 #include <QDebug>
 #include <QProcess>
 #include <QFileInfo>
+#include <QCoreApplication>
+
+#include "staticplugins.cpp"
 
 namespace Avogadro {
 
@@ -222,7 +221,12 @@ namespace Avogadro {
     else if (second->identifier() == "ElementColor")
       return false; // always the top!
 
-    // locale aware returns less-than, greater-than, or 0 
+    if (first->identifier() == "CustomColor")
+      return false; // always the bottom!
+    else if (second->identifier() == "CustomColor")
+      return true; // always the bottom!
+
+    // locale aware returns less-than, greater-than, or 0
     // Required for sorting lists.
     return (QString::localeAwareCompare(first->name(), second->name()) < 0);
   }
@@ -230,35 +234,6 @@ namespace Avogadro {
   PluginManager::PluginManager(QObject *parent) : QObject(parent),
                                                   d(new PluginManagerPrivate)
   {
-    // This is where we compile a list of directories that will be searched
-    // for plugins. This is quite dependent up on operating system
-
-    // Environment variables can override default paths
-    foreach (const QString &variable, QProcess::systemEnvironment()) {
-      QStringList split1 = variable.split('=');
-      if (split1[0] == "AVOGADRO_PLUGINS")
-        foreach (const QString &path, split1[1].split(':'))
-          d->searchDirs << path;
-    }
-    // If no environment variables are set then find the plugins
-    if (!d->searchDirs.size()) {
-      // Make it relative
-      d->searchDirs << QCoreApplication::applicationDirPath()
-                     + "/../" + QString(INSTALL_LIB_DIR)
-                     + "/" + QString(INSTALL_PLUGIN_DIR);
-    }
-
-    // Now to search for the plugins in home directories
-#if defined(Q_WS_X11)
-    d->searchDirs << QDir::homePath() + "/."
-                   + QString(INSTALL_PLUGIN_DIR) + "/plugins";
-#elif defined(Q_WS_MAC)
-    d->searchDirs << QDir::homePath() + "/Library/Application Support/"
-                   + QString(INSTALL_PLUGIN_DIR) + "/Plugins";
-#elif defined(WIN32)
-    const QString appdata = qgetenv("APPDATA");
-    d->searchDirs << appdata + "/" + QString(INSTALL_PLUGIN_DIR);
-#endif
   }
 
   PluginManager::~PluginManager()
@@ -278,6 +253,31 @@ namespace Avogadro {
     return obj;
   }
 
+  void PluginManager::setPluginPath(const QString &path)
+  {
+    d->searchDirs = QStringList(path);
+  }
+
+  void PluginManager::setPluginPath(const QStringList &path)
+  {
+    d->searchDirs = path;
+  }
+
+  void PluginManager::addToPluginPath(const QString &path)
+  {
+    d->searchDirs << path;
+  }
+
+  void PluginManager::addToPluginPath(const QStringList &path)
+  {
+    d->searchDirs << path;
+  }
+
+  QStringList PluginManager::pluginPath() const
+  {
+    return d->searchDirs;
+  }
+
   QList<Extension *> PluginManager::extensions(QObject *parent)
   {
     loadFactories();
@@ -285,12 +285,11 @@ namespace Avogadro {
       return d->extensions;
 
     foreach(PluginFactory *factory, factories(Plugin::ExtensionType)) {
-      Extension *extension = static_cast<Extension *>(factory->createInstance(parent));
+      Extension *extension =
+          static_cast<Extension *>(factory->createInstance(parent));
       d->extensions.append(extension);
     }
 
-    //    qSort(d->extensions.begin(), d->extensions.end(), extensionGreaterThan);
-
     d->extensionsLoaded = true;
 
     return d->extensions;
@@ -347,6 +346,11 @@ namespace Avogadro {
       if (!dir.cd(type + "Scripts")) failed = true;
     }
 
+    if (failed) {
+      qWarning() << "Insufficient permissions to access scripts directory";
+      return scripts;
+    }
+
     foreach (const QString& file, dir.entryList())
       scripts.append(QString(dir.canonicalPath() + '/' + file));
 
@@ -537,46 +541,31 @@ namespace Avogadro {
       settings.endGroup(); // ExtraPlugins
     }
 
-    QVector< QList<PluginFactory *> > &ef = PluginManagerPrivate::m_enabledFactories();
+    // Initialize the resource files for our static plugins. Note that the
+    // function must be declared outside of a namespace.
+    initAvogadroResources();
 
-    // Load the static plugins first
-    PluginFactory *bsFactory = qobject_cast<PluginFactory *>(new BSDYEngineFactory);
-    if (bsFactory)
-      ef[bsFactory->type()].append(bsFactory);
-    else
-      qDebug() << "Instantiation of the static ball and sticks plugin failed.";
-
-    PluginFactory *elementFactory = qobject_cast<PluginFactory *>(new ElementColorFactory);
-    if (elementFactory)
-      ef[elementFactory->type()].append(elementFactory);
-    else
-      qDebug() << "Instantiation of the static element color plugin failed.";
+    // Load the static plugins
+    QVector<QList<PluginFactory *> > &ef =
+      PluginManagerPrivate::m_enabledFactories();
+    foreach(QObject *instance, QPluginLoader::staticInstances()) {
+      PluginFactory *factory = qobject_cast<PluginFactory *>(instance);
+      if (factory)
+        ef[factory->type()].append(factory);
+    }
 
     QSettings settings;
     settings.beginGroup("Plugins");
 
-#ifndef Q_WS_MAC
-    QFileInfo info(QCoreApplication::applicationDirPath()
-                   + "/../CMakeCache.txt");
-    if (info.exists()) {// In a build directory
-      qDebug() << "In a build directory - loading alternative...";
-      loadPluginDir(QCoreApplication::applicationDirPath() + "/../lib",
-                    settings);
-    }
-#else
-    // If we are in a Mac build dir things are a little different - if the
-    // expected relative path does not exist try the build dir path
-    QFileInfo info(QCoreApplication::applicationDirPath()
-                   + "/../CMakeCache.txt");
-    if (info.exists()) {// In a build directory
-        loadPluginDir(QCoreApplication::applicationDirPath()
-                      + "/../../../../lib", settings);
-    }
-#endif
+    // Initialize search path if it was not done by client application
+    /// @todo To be removed in libavogadro 2
+    if(d->searchDirs.empty())
+      initializeSearchDirs(d->searchDirs);
 
     // Load the plugins
     foreach (const QString& path, d->searchDirs) {
       qDebug() << "Loading plugins:" << path;
+      loadPluginDir(path, settings);
       loadPluginDir(path + "/colors", settings);
       loadPluginDir(path + "/engines", settings);
       loadPluginDir(path + "/extensions", settings);
@@ -633,18 +622,22 @@ namespace Avogadro {
   {
     settings.beginGroup(QString::number(factory->type()));
 
-    QVector< QList<PluginFactory *> > &ef = PluginManagerPrivate::m_enabledFactories();
-    QVector< QList<PluginFactory *> > &df = PluginManagerPrivate::m_disabledFactories();
+    QVector< QList<PluginFactory *> > &ef =
+        PluginManagerPrivate::m_enabledFactories();
+    QVector< QList<PluginFactory *> > &df =
+        PluginManagerPrivate::m_disabledFactories();
 
     // create the PluginItem
     PluginItem *item = new PluginItem(factory->name(), factory->identifier(),
                                       factory->description(),
-        factory->type(), fileInfo.fileName(), fileInfo.absoluteFilePath(), factory);
+                                      factory->type(), fileInfo.fileName(),
+                                      fileInfo.absoluteFilePath(), factory);
     // add the factory to the correct list
     if(settings.value(factory->identifier(), true).toBool()) {
       ef[factory->type()].append(factory);
       item->setEnabled(true);
-    } else {
+    }
+    else {
       df[factory->type()].append(factory);
       item->setEnabled(false);
     }
@@ -760,17 +753,29 @@ namespace Avogadro {
   inline void PluginManager::loadPluginDir(const QString &directory,
                                            QSettings &settings)
   {
-    QDir dir(directory);
+    const QDir dir(directory);
+    if(dir.exists()) {
+      qDebug() << "Searching for plugins in" << dir.canonicalPath();
+      loadPluginList(dir, dir.entryList(QDir::Files), settings);
+    }
+  }
+
+  void PluginManager::loadPluginList(const QDir &dir,
+                                const QStringList &plugins, QSettings &settings)
+  {
+    foreach (const QString& fileName, plugins) {
+      if(!QLibrary::isLibrary(fileName))
+        continue;
 #ifdef Q_WS_X11
-    QStringList dirFilters;
-    dirFilters << "*.so";
-    dir.setNameFilters(dirFilters);
-    dir.setFilter(QDir::Files | QDir::Readable);
+      if ((fileName.indexOf("libavogadro.so") != -1)
+        || (fileName.indexOf("Avogadro.so") != -1)
+        || (fileName.indexOf("libQPeriodicTable.so") != -1)
+        || (fileName.indexOf("libQPlotWidget.so") != -1))
+          continue;
 #endif
-    qDebug() << "Searching for plugins in" << directory;
-    foreach (const QString& fileName, dir.entryList(QDir::Files)) {
-#ifdef Q_WS_X11
-      if ((fileName.indexOf("libavogadro.so") != -1) || (fileName.indexOf("Avogadro.so") != -1))
+
+#ifndef ENABLE_PYTHON
+      if (fileName.indexOf("pythonterminal") != -1)
         continue;
 #endif
       // load the factory
@@ -787,6 +792,38 @@ namespace Avogadro {
     }
   }
 
+  void PluginManager::initializeSearchDirs(QStringList &searchDirs)
+  {
+    qDebug() << "PluginManager::setPluginPath() was not called from application"
+                " - using default plugin path";
+    // Environment variable overrides default paths
+    foreach (const QString &variable, QProcess::systemEnvironment()) {
+      if(variable.startsWith("AVOGADRO_PLUGINS=")) {
+        QString path(variable);
+        path.remove(QRegExp("^AVOGADRO_PLUGINS="));
+        searchDirs << path.split(':');
+      }
+    }
+    // If no environment variables are set then find the plugins
+    if (searchDirs.isEmpty()) {
+      // Make it relative
+      searchDirs << QCoreApplication::applicationDirPath()
+                       + "/../" + QString(INSTALL_LIB_DIR)
+                       + "/" + QString(INSTALL_PLUGIN_DIR);
+    }
+
+    // Now to search for the plugins in home directories
+  #if defined(Q_WS_X11)
+    searchDirs << QDir::homePath() + "/."
+                     + QString(INSTALL_PLUGIN_DIR) + "/plugins";
+  #elif defined(Q_WS_MAC)
+    searchDirs << QDir::homePath() + "/Library/Application Support/"
+                     + QString(INSTALL_PLUGIN_DIR) + "/Plugins";
+  #elif defined(WIN32)
+    const QString appdata = qgetenv("APPDATA");
+    searchDirs << appdata + "/" + QString(INSTALL_PLUGIN_DIR);
+  #endif
+  }
+
 }
 
-#include "pluginmanager.moc"
