@@ -977,6 +977,4 @@ namespace Avogadro {
     }
   }
 
-} // End namespace Avogadro
-
-#include "protein.moc"
+} // End namespace
