@@ -44,8 +44,8 @@
 
 #define AVOGADRO_TOOL_FACTORY(c)                             \
   public: \
-    Plugin *createInstance(QObject *parent = 0) { return new c(parent); } \
-    Plugin::Type type() const { return Plugin::ToolType; }   \
+    Avogadro::Plugin *createInstance(QObject *parent = 0) { return new c(parent); } \
+    Avogadro::Plugin::Type type() const { return Avogadro::Plugin::ToolType; } \
     QString identifier() const { return c::staticIdentifier(); } \
     QString name() const { return c::staticName(); }         \
     QString description() const { return c::staticDescription(); }
@@ -130,6 +130,14 @@ namespace Avogadro {
        */
       virtual QUndoCommand* mouseMoveEvent(GLWidget *widget, QMouseEvent *event) = 0;
 
+      /**
+       * Response to a user double-click
+       * @param widget the %GLWidget where the event occurred
+       * @param event the mouse event information
+       * @since version 1.1
+       */
+      virtual QUndoCommand* mouseDoubleClickEvent(GLWidget *widget, QMouseEvent *event) = 0;
+
       /**
        * Response to mouse wheel movement
        * @param widget the %GLWidget where the event occurred
