@@ -254,7 +254,7 @@ namespace Avogadro {
       foreach (unsigned long id, d->atomIds) {
         Atom *atom = d->molecule->atomById(id);
         Q_CHECK_PTR( atom );
-        
+
         if (atom) {
           if (atom->isHydrogen()) {
             qDebug() << "AdjustHydrogensPostCommand::redo(): Error, request to add hydrogens on hydrogen atom";
@@ -336,7 +336,7 @@ namespace Avogadro {
     Q_CHECK_PTR( atom );
     if (atom) {
       // Remove the previously add hydrogens if needed
-      if (d->adjustHydrogens & AdjustHydrogens::RemoveOnUndo)
+      if (d->adjustHydrogens & AdjustHydrogens::RemoveOnUndo && !atom->isHydrogen())
         d->postCommand->undo();
 
       d->molecule->removeAtom(atom);
@@ -348,12 +348,11 @@ namespace Avogadro {
   void AddAtomDrawCommand::redo()
   {
     if (d->atom) { // initial creation
-      if (d->adjustHydrogens != AdjustHydrogens::Never) {
+      if (d->adjustHydrogens != AdjustHydrogens::Never && !d->atom->isHydrogen()) {
         d->postCommand = new AdjustHydrogensPostCommand(d->molecule, d->id);
         if (d->adjustHydrogens & AdjustHydrogens::AddOnRedo)
           d->postCommand->redo();
       }
-      d->atom->update();
       d->atom = 0;
 #ifdef DEBUG_COMMANDS
       qDebug() << "AddAtomDrawCommand::redo(id = " << d->id << ")";
@@ -373,7 +372,7 @@ namespace Avogadro {
     atom->setPos(d->pos);
     atom->setAtomicNumber(d->element);
 
-    if (d->adjustHydrogens != AdjustHydrogens::Never) {
+    if (d->adjustHydrogens != AdjustHydrogens::Never && !atom->isHydrogen()) {
       if (!d->postCommand)
         d->postCommand = new AdjustHydrogensPostCommand(d->molecule, d->id);
       if (d->adjustHydrogens & AdjustHydrogens::AddOnRedo)
@@ -384,7 +383,7 @@ namespace Avogadro {
     qDebug() << "AddAtomDrawCommand::redo(id = " << d->id << ")";
 #endif
 
-    atom->update();
+    d->molecule->update();
   }
 
   /////////////////////////////////////////////////////////////////////////////
@@ -403,13 +402,13 @@ namespace Avogadro {
       QList<unsigned long> neighbors;
       Eigen::Vector3d pos;
       unsigned int element;
-      int adjustHydrogens;
+      bool adjustHydrogens;
 
       QUndoCommand *preCommand;
       QUndoCommand *postCommand;
   };
 
-  DeleteAtomDrawCommand::DeleteAtomDrawCommand(Molecule *molecule, int index, int adjustHydrogens)
+  DeleteAtomDrawCommand::DeleteAtomDrawCommand(Molecule *molecule, int index, bool adjustHydrogens)
       : d(new DeleteAtomDrawCommandPrivate)
   {
     setText(QObject::tr("Delete Atom"));
@@ -488,13 +487,14 @@ namespace Avogadro {
     if (atom) {
       QList<unsigned long> neighbors;
 
-      if (d->adjustHydrogens) {
+      if (d->adjustHydrogens && !atom->isHydrogen()) {
         if (!d->preCommand) {
           QList<unsigned long> ids;
           foreach (unsigned long id, atom->neighbors()) {
-            if (!d->molecule->atomById(id)->isHydrogen())
+            if (!d->molecule->atomById(id)->isHydrogen()) {
               neighbors.append(id);
               ids.append(id);
+            }
           }
 
           ids.append(atom->id());
@@ -508,7 +508,7 @@ namespace Avogadro {
       // Delete the atom, also deletes the bonds
       d->molecule->removeAtom(atom);
 
-      if (d->adjustHydrogens) {
+      if (d->adjustHydrogens && !atom->isHydrogen()) {
         if (!d->postCommand)
           // Add hydrogens to the heavy atom neighbors of the delted atom
           // Uses new ids the first time, reuses these on successive calls
@@ -674,6 +674,7 @@ namespace Avogadro {
       }
 
       d->bond = 0;
+      d->molecule->update();
       return;
     }
 
@@ -700,14 +701,17 @@ namespace Avogadro {
     Atom *endAtom = d->molecule->atomById(d->endAtomId);
     Q_CHECK_PTR( endAtom );
 
-    if (!beginAtom || !endAtom)
+    if (!beginAtom || !endAtom) {
+      d->molecule->update();
       return;
+    }
 
     Bond *bond;
     if (d->id != FALSE_ID) {
       bond = d->molecule->addBond(d->id);
       Q_CHECK_PTR( bond );
-    } else {
+    }
+    else {
       bond = d->molecule->addBond();
       Q_CHECK_PTR( bond );
       d->id = bond->id();
@@ -749,10 +753,10 @@ namespace Avogadro {
       Molecule *molecule;
       Molecule moleculeCopy;
       unsigned long id;
-      int adjustHydrogens;
+      bool adjustHydrogens;
   };
 
-  DeleteBondDrawCommand::DeleteBondDrawCommand(Molecule *molecule, int index, int adjustHydrogens) : d(new DeleteBondDrawCommandPrivate)
+  DeleteBondDrawCommand::DeleteBondDrawCommand(Molecule *molecule, int index, bool adjustHydrogens) : d(new DeleteBondDrawCommandPrivate)
   {
     setText(QObject::tr("Delete Bond"));
     d->molecule = molecule;
@@ -816,17 +820,17 @@ namespace Avogadro {
       Molecule *molecule;
       unsigned int newElement, oldElement;
       unsigned long id;
-      int adjustHydrogens;
+      bool adjustHydrogens;
 
       QUndoCommand *preCommand;
       QUndoCommand *postCommand;
   };
 
   ChangeElementDrawCommand::ChangeElementDrawCommand(Molecule *molecule, Atom *atom, unsigned int oldElement,
-      int adjustHydrogens) : d(new ChangeElementDrawCommandPrivate)
+      bool adjustHydrogens) : d(new ChangeElementDrawCommandPrivate)
   {
 #ifdef DEBUG_COMMANDS
-    qDebug() << "ChangeElementDrawCommand(id = " << atom->id() << ", old = " << oldElement 
+    qDebug() << "ChangeElementDrawCommand(id = " << atom->id() << ", old = " << oldElement
              << ", new = " << atom->atomicNumber() << ", adj=" << adjustHydrogens << ")";
 #endif
 
@@ -838,7 +842,7 @@ namespace Avogadro {
     d->adjustHydrogens = adjustHydrogens;
   }
     
-  void ChangeElementDrawCommand::setAdjustHydrogens(int adjustHydrogens)
+  void ChangeElementDrawCommand::setAdjustHydrogens(bool adjustHydrogens)
   {
     d->adjustHydrogens = adjustHydrogens;
   }
@@ -891,25 +895,27 @@ namespace Avogadro {
     if (atom) {
       // Remove Hydrogens if needed
       if (d->adjustHydrogens) {
-        if (!d->preCommand) {
+        if (!atom->isHydrogen() && !d->preCommand) {
           QList<unsigned long> ids;
           ids.append(d->id);
           d->preCommand = new AdjustHydrogensPreCommand(d->molecule, ids);
         }
-        d->preCommand->redo();
+        if (d->preCommand)
+          d->preCommand->redo();
       }
 
       // Make sure we call BeginModify / EndModify (e.g., PR#1720879)
       atom->setAtomicNumber(d->newElement);
 
       // Add hydrogens again if needed
-      if (d->adjustHydrogens) {
+      if (!atom->isHydrogen() && d->adjustHydrogens) {
         if (!d->postCommand) {
           QList<unsigned long> ids;
           ids.append(d->id);
           d->postCommand = new AdjustHydrogensPostCommand(d->molecule, ids);
         }
-        d->postCommand->redo();
+        if (d->postCommand)
+          d->postCommand->redo();
       }
 
       d->molecule->update();
@@ -928,14 +934,14 @@ namespace Avogadro {
       Molecule *molecule;
       unsigned long id;
       unsigned int addBondOrder, oldBondOrder;
-      int adjustHydrogens;
+      bool adjustHydrogens;
 
       QUndoCommand *preCommand;
       QUndoCommand *postCommand;
   };
 
   ChangeBondOrderDrawCommand::ChangeBondOrderDrawCommand(Molecule *molecule, Bond *bond,
-      unsigned int oldBondOrder, int adjustHydrogens) : d(new ChangeBondOrderDrawCommandPrivate)
+      unsigned int oldBondOrder, bool adjustHydrogens) : d(new ChangeBondOrderDrawCommandPrivate)
   {
 #ifdef DEBUG_COMMANDS
     qDebug() << "ChangeBondOrderDrawCommand(id = " << bond->id() << ", old = " << oldBondOrder
@@ -1023,42 +1029,4 @@ namespace Avogadro {
     }
   }
 
-  /////////////////////////////////////////////////////////////////////////////
-  // Insert Fragment
-  /////////////////////////////////////////////////////////////////////////////
-
-  class InsertFragmentCommandPrivate {
-    public:
-      InsertFragmentCommandPrivate() : molecule(0), generatedMolecule(0) {};
-
-      Molecule *molecule;
-      Molecule moleculeCopy, generatedMolecule;
-  };
-
-  InsertFragmentCommand::InsertFragmentCommand(Molecule *molecule, Molecule &generatedMolecule)
-      : d(new InsertFragmentCommandPrivate)
-  {
-    setText(QObject::tr("Insert Fragment"));
-    d->molecule = molecule;
-    d->moleculeCopy = *molecule;
-    d->generatedMolecule = generatedMolecule;
-  }
-
-  InsertFragmentCommand::~InsertFragmentCommand()
-  {
-    delete d;
-  }
-
-  void InsertFragmentCommand::undo()
-  {
-    *(d->molecule) = d->moleculeCopy;
-    d->molecule->update();
-  }
-
-  void InsertFragmentCommand::redo()
-  {
-    *(d->molecule) += d->generatedMolecule;
-    d->molecule->update();
-  }
-
 } // end namespace Avogadro
