@@ -2,7 +2,7 @@
   ManipulateTool - Manipulation Tool for Avogadro
 
   Copyright (C) 2007 by Marcus D. Hanwell
-  Copyright (C) 2007 by Geoffrey R. Hutchison
+  Copyright (C) 2007,2011 by Geoffrey R. Hutchison
   Copyright (C) 2007 by Benoit Jacob
 
   This file is part of the Avogadro molecular editor project.
@@ -23,6 +23,9 @@
 
 #include "manipulatetool.h"
 #include "eyecandy.h"
+#include "moveatomcommand.h"
+
+#include "ui_manipulatesettingswidget.h"
 
 #include <avogadro/navigate.h>
 #include <avogadro/atom.h>
@@ -34,6 +37,7 @@
 #include <avogadro/camera.h>
 
 #include <QtPlugin>
+#include <QAbstractButton>
 
 using Eigen::Vector3d;
 using Eigen::Transform3d;
@@ -41,17 +45,32 @@ using Eigen::AngleAxisd;
 
 namespace Avogadro {
 
+  class ManipulateSettingsWidget : public QWidget,
+                                   public Ui::ManipulateSettingsWidget
+  {
+  public:
+    ManipulateSettingsWidget(QWidget *parent=0) : QWidget(parent)
+    {
+      setupUi(this);
+    }
+  };
+
   ManipulateTool::ManipulateTool(QObject *parent) : Tool(parent),
-    m_clickedAtom(0), m_leftButtonPressed(false), m_midButtonPressed(false),
-    m_rightButtonPressed(false), m_eyecandy(new Eyecandy)
+                                                    m_clickedAtom(0),
+                                                    m_leftButtonPressed(false),
+                                                    m_midButtonPressed(false),
+                                                    m_rightButtonPressed(false),
+                                                    m_eyecandy(new Eyecandy),
+                                                    m_settingsWidget(0)
   {
     m_eyecandy->setColor(1.0, 0.0, 0.0, 1.0);
     QAction *action = activateAction();
     action->setIcon(QIcon(QString::fromUtf8(":/manipulate/manipulate.png")));
     action->setToolTip(tr("Manipulation Tool (F10)\n\n"
-          "Left Mouse:   Click and drag to move atoms\n"
+          "Left Mouse: \tClick and drag to move atoms\n"
           "Middle Mouse: Click and drag to move atoms further away or closer\n"
-          "Right Mouse:  Click and drag to rotate selected atoms."));
+          "Right Mouse: \tClick and drag to rotate selected atoms.\n"
+          "Double-Click: \tReset the view."));
     action->setShortcut(Qt::Key_F10);
   }
 
@@ -65,7 +84,115 @@ namespace Avogadro {
     return 600000;
   }
 
-  void ManipulateTool::zoom(GLWidget *widget, const Eigen::Vector3d *goal,
+  QWidget* ManipulateTool::settingsWidget()
+  {
+    if (!m_settingsWidget) {
+      m_settingsWidget = new ManipulateSettingsWidget(qobject_cast<QWidget*>(parent()));
+      // each time, we should reset to 0 values
+      connect(m_settingsWidget->buttonBox, SIGNAL(clicked(QAbstractButton*)),
+              this, SLOT(buttonClicked(QAbstractButton*)));
+    }
+    return m_settingsWidget;
+  }
+
+  void ManipulateTool::applyManualManipulation()
+  {
+    if (!m_settingsWidget)
+      return;
+
+    // Before doing anything, save an undo state
+    // Get the current GLWidget for the manipulation
+    GLWidget *widget = GLWidget::current();
+    QUndoStack *stack = widget->undoStack();
+    QUndoCommand* undo = new MoveAtomCommand(widget->molecule());
+    if(stack && undo)
+        stack->push(undo);
+
+    // Get translations and rotations
+    double x = m_settingsWidget->xTranslateSpinBox->value();
+    double y = m_settingsWidget->yTranslateSpinBox->value();
+    double z = m_settingsWidget->zTranslateSpinBox->value();
+    Eigen::Vector3d translate(x, y, z);
+    Eigen::Vector3d center(0.0, 0.0, 0.0);
+
+    // Check if we're rotating around the origin or the centroid
+    if (m_settingsWidget->rotateComboBox->currentIndex() == 1) {
+      if (widget->selectedPrimitives().size()) {
+        foreach(Primitive *p, widget->selectedPrimitives())
+          if (p->type() == Primitive::AtomType) {
+            Atom *atom = static_cast<Atom*>(p);
+            center += *(atom->pos());
+          }
+        center /= widget->selectedPrimitives().size();
+      }
+      else {
+        center = widget->molecule()->center();
+      }
+    }
+
+    // Settings are in degrees
+#ifndef DEG_TO_RAD
+#define DEG_TO_RAD 0.0174532925
+#endif
+    double xRotate = m_settingsWidget->xRotateSpinBox->value() * DEG_TO_RAD;
+    double yRotate = m_settingsWidget->yRotateSpinBox->value() * DEG_TO_RAD;
+    double zRotate = m_settingsWidget->zRotateSpinBox->value() * DEG_TO_RAD;
+
+    Eigen::Transform3d rotation;
+    rotation.matrix().setIdentity();
+    rotation.translation() = center;
+    rotation.rotate(AngleAxisd(xRotate, Vector3d::UnitX())
+                            * AngleAxisd(yRotate, Vector3d::UnitY())
+                            * AngleAxisd(zRotate, Vector3d::UnitZ()));
+    rotation.translate(- center);
+
+    Eigen::Vector3d tempPos; // from the translation
+    if (widget->selectedPrimitives().size()) {
+      foreach(Primitive *p, widget->selectedPrimitives())
+        if (p->type() == Primitive::AtomType) {
+          Atom *atom = static_cast<Atom*>(p);
+          tempPos = translate + *(atom->pos());
+          atom->setPos(rotation * tempPos);
+        }
+    } else {
+      foreach(Atom *atom, widget->molecule()->atoms()) {
+        tempPos = translate + *(atom->pos());
+        atom->setPos(rotation * tempPos);
+      }
+    }
+
+    widget->molecule()->update();
+  }
+
+  void ManipulateTool::buttonClicked(QAbstractButton *button)
+  {
+    if (!m_settingsWidget)
+      return;
+
+    if (m_settingsWidget->buttonBox->buttonRole(button) == QDialogButtonBox::ApplyRole) {
+      applyManualManipulation();
+    } else {
+      // reset values
+      m_settingsWidget->xTranslateSpinBox->setValue(0.0);
+      m_settingsWidget->yTranslateSpinBox->setValue(0.0);
+      m_settingsWidget->zTranslateSpinBox->setValue(0.0);
+
+      m_settingsWidget->xRotateSpinBox->setValue(0.0);
+      m_settingsWidget->yRotateSpinBox->setValue(0.0);
+      m_settingsWidget->zRotateSpinBox->setValue(0.0);
+    }
+
+    // clear focus from the boxes (they eat up keystrokes)
+    m_settingsWidget->xTranslateSpinBox->clearFocus();
+    m_settingsWidget->yTranslateSpinBox->clearFocus();
+    m_settingsWidget->zTranslateSpinBox->clearFocus();
+
+    m_settingsWidget->xRotateSpinBox->clearFocus();
+    m_settingsWidget->yRotateSpinBox->clearFocus();
+    m_settingsWidget->zRotateSpinBox->clearFocus();
+  }
+
+  void ManipulateTool::zoom(GLWidget *widget, const Vector3d *goal,
                             double delta) const
   {
     // Set the cursor - this needs to be reset to Qt::ArrowCursor after
@@ -356,56 +483,6 @@ namespace Avogadro {
     return true;
   }
 
-  MoveAtomCommand::MoveAtomCommand(Molecule *molecule, QUndoCommand *parent) : QUndoCommand(parent), m_molecule(0)
-  {
-    // Store the molecule - this call won't actually move an atom
-    setText(QObject::tr("Manipulate Atom"));
-    m_moleculeCopy = *molecule;
-    m_molecule = molecule;
-    undone = false;
-  }
-
-  MoveAtomCommand::MoveAtomCommand(Molecule *molecule, int type, QUndoCommand *parent) : QUndoCommand(parent), m_molecule(0)
-  {
-    // Store the original molecule before any modifications are made
-    setText(QObject::tr("Manipulate Atom"));
-    m_moleculeCopy = *molecule;
-    m_molecule = molecule;
-    m_type =type;
-    undone = false;
-  }
-
-  void MoveAtomCommand::redo()
-  {
-    // Move the specified atom to the location given
-    if (undone)
-    {
-      Molecule newMolecule = *m_molecule;
-      *m_molecule = m_moleculeCopy;
-      m_moleculeCopy = newMolecule;
-    }
-    QUndoCommand::redo();
-  }
-
-  void MoveAtomCommand::undo()
-  {
-    // Restore our original molecule
-    Molecule newMolecule = *m_molecule;
-    *m_molecule = m_moleculeCopy;
-    m_moleculeCopy = newMolecule;
-    undone = true;
-  }
-
-  bool MoveAtomCommand::mergeWith (const QUndoCommand *)
-  {
-    // Just return true to repeated calls - we have stored the original molecule
-    return true;
-  }
-
-  int MoveAtomCommand::id() const
-  {
-    return 26011980;
-  }
-}
+} // end namespace Avogadro
 
 Q_EXPORT_PLUGIN2(manipulatetool, Avogadro::ManipulateToolFactory)
